@@ -1,12 +1,19 @@
-from typing import List, Optional
+import ssl
+import typing
 
 import hpack
 import hyperframe.frame
 import pytest
 
-from httpcore import AsyncHTTPConnection, ConnectError, ConnectionNotAvailable, Origin
-from httpcore.backends.base import AsyncNetworkStream
-from httpcore.backends.mock import AsyncMockBackend
+from httpcore import (
+    SOCKET_OPTION,
+    AsyncHTTPConnection,
+    AsyncMockBackend,
+    AsyncNetworkStream,
+    ConnectError,
+    ConnectionNotAvailable,
+    Origin,
+)
 
 
 @pytest.mark.anyio
@@ -124,24 +131,69 @@ async def test_request_to_incorrect_origin():
 
 
 class NeedsRetryBackend(AsyncMockBackend):
-    def __init__(self, buffer: List[bytes], http2: bool = False) -> None:
-        self._retry = 2
+    def __init__(
+        self,
+        buffer: typing.List[bytes],
+        http2: bool = False,
+        connect_tcp_failures: int = 2,
+        start_tls_failures: int = 0,
+    ) -> None:
+        self._connect_tcp_failures = connect_tcp_failures
+        self._start_tls_failures = start_tls_failures
         super().__init__(buffer, http2)
 
     async def connect_tcp(
         self,
         host: str,
         port: int,
-        timeout: Optional[float] = None,
-        local_address: Optional[str] = None,
+        timeout: typing.Optional[float] = None,
+        local_address: typing.Optional[str] = None,
+        socket_options: typing.Optional[typing.Iterable[SOCKET_OPTION]] = None,
     ) -> AsyncNetworkStream:
-        if self._retry > 0:
-            self._retry -= 1
+        if self._connect_tcp_failures > 0:
+            self._connect_tcp_failures -= 1
             raise ConnectError()
 
-        return await super().connect_tcp(
+        stream = await super().connect_tcp(
             host, port, timeout=timeout, local_address=local_address
         )
+        return self._NeedsRetryAsyncNetworkStream(self, stream)
+
+    class _NeedsRetryAsyncNetworkStream(AsyncNetworkStream):
+        def __init__(
+            self, backend: "NeedsRetryBackend", stream: AsyncNetworkStream
+        ) -> None:
+            self._backend = backend
+            self._stream = stream
+
+        async def read(
+            self, max_bytes: int, timeout: typing.Optional[float] = None
+        ) -> bytes:
+            return await self._stream.read(max_bytes, timeout)
+
+        async def write(
+            self, buffer: bytes, timeout: typing.Optional[float] = None
+        ) -> None:
+            await self._stream.write(buffer, timeout)
+
+        async def aclose(self) -> None:
+            await self._stream.aclose()
+
+        async def start_tls(
+            self,
+            ssl_context: ssl.SSLContext,
+            server_hostname: typing.Optional[str] = None,
+            timeout: typing.Optional[float] = None,
+        ) -> "AsyncNetworkStream":
+            if self._backend._start_tls_failures > 0:
+                self._backend._start_tls_failures -= 1
+                raise ConnectError()
+
+            stream = await self._stream.start_tls(ssl_context, server_hostname, timeout)
+            return self._backend._NeedsRetryAsyncNetworkStream(self._backend, stream)
+
+        def get_extra_info(self, info: str) -> typing.Any:
+            return self._stream.get_extra_info(info)
 
 
 @pytest.mark.anyio
@@ -171,6 +223,37 @@ async def test_connection_retries():
             await conn.request("GET", "https://example.com/")
 
 
+@pytest.mark.anyio
+async def test_connection_retries_tls():
+    origin = Origin(b"https", b"example.com", 443)
+    content = [
+        b"HTTP/1.1 200 OK\r\n",
+        b"Content-Type: plain/text\r\n",
+        b"Content-Length: 13\r\n",
+        b"\r\n",
+        b"Hello, world!",
+    ]
+
+    network_backend = NeedsRetryBackend(
+        content, connect_tcp_failures=0, start_tls_failures=2
+    )
+    async with AsyncHTTPConnection(
+        origin=origin, network_backend=network_backend, retries=3
+    ) as conn:
+        response = await conn.request("GET", "https://example.com/")
+        assert response.status == 200
+
+    network_backend = NeedsRetryBackend(
+        content, connect_tcp_failures=0, start_tls_failures=2
+    )
+    async with AsyncHTTPConnection(
+        origin=origin,
+        network_backend=network_backend,
+    ) as conn:
+        with pytest.raises(ConnectError):
+            await conn.request("GET", "https://example.com/")
+
+
 @pytest.mark.anyio
 async def test_uds_connections():
     # We're not actually testing Unix Domain Sockets here, because we're just
